from django.urls import path
from django.utils.translation import gettext_lazy as _

from basxbread import menu, utils

from . import views

urlpatterns = [
    path("test", views.test, name="modeledit.test"),
]

menu.registeritem(
    menu.Item(
        utils.Link(
            utils.reverse("modeledit.test"),
            _("Modeledit"),
        ),
        _("Modeledit"),
    )
)



# Generated by Django 4.0.5 on 2022-09-07 03:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reports', '0011_reportcolumn_cell_template'),
    ]

    operations = [
        migrations.AddField(
            model_name='reportcolumn',
            name='allow_html',
            field=models.BooleanField(default=False, help_text='Do render HTML code inside the cell template instead of escaping it.', verbose_name='Allow HTML'),
        ),
        migrations.AlterField(
            model_name='reportcolumn',
            name='cell_template',
            field=models.TextField(blank=True, help_text="Optional Jinja template with 'value' in context.<br/>Use e.g. {{ value }} to simply display the value.", verbose_name='Cell template'),
        ),
    ]



import htmlgenerator as hg
from django.utils.translation import gettext_lazy as _

from .button import Button
from .icon import Icon


def asoverflowbutton(context):
    link = context["link"]

    # This is not working, overflow-menus with link.is_submit will not work
    if link.is_submit:
        from django.forms import Form as DjangoForm

        from ..utils import slugify
        from .forms import Form
        from .modal import Modal

        confirm_dialog = Modal(
            _("Please confirm"),
            link.confirm_text,
            buttons=(
                Button(_("No..."), buttontype="ghost", data_modal_close=True),
                Button(_("Yes!"), type="submit"),
            ),
            id=hg.format("modal-{}", slugify(link.href)),
            size="xs",
        )

        return hg.BaseElement(
            Form(
                DjangoForm(),
                hg.DIV(
                    hg.If(
                        link.iconname,
                        Icon(link.iconname, size=16),
                    ),
                    link.label,
                    _class="bx--overflow-menu-options__option-content",
                ),
                action=link.href,
                _class="bx--overflow-menu-options__btn",
                role="menuitem",
                title=link.label,
                **hg.merge_html_attrs(
                    link.attributes,
                    {
                        **confirm_dialog.openerattributes,
                        "style": "text-decoration: underline",
                    },
                ),
            ),
            confirm_dialog,
        )
    else:
        return hg.A(
            hg.DIV(
                hg.If(
                    link.iconname,
                    Icon(link.iconname, size=16),
                ),
                link.label,
                _class="bx--overflow-menu-options__option-content",
            ),
            _class="bx--overflow-menu-options__btn",
            role="menuitem",
            title=link.label,
            href=link.href,
            **link.attributes,
        )


class OverflowMenu(hg.DIV):
    """Implements https://www.carbondesignsystem.com/components/overflow-menu/usage"""

    MENUID_TEMPLATE = "overflow-menu-%s"

    def __init__(
        self,
        links,
        menuiconname="overflow-menu--vertical",
        menuname=None,
        direction="bottom",
        flip=False,
        item_attributes={},
        **attributes,
    ):
        attributes["data-overflow-menu"] = True
        attributes["_class"] = attributes.get("_class", "") + " bx--overflow-menu"
        item_attributes["_class"] = (
            item_attributes.get("_class", "") + " bx--overflow-menu-options__option"
        )

        menuid = hg.F(
            lambda c: OverflowMenu.MENUID_TEMPLATE % hg.html_id(c.get("row", self))
        )
        triggerid = hg.F(
            lambda c: (OverflowMenu.MENUID_TEMPLATE % hg.html_id(c.get("row", self)))
            + "-trigger"
        )

        super().__init__(
            hg.BUTTON(
                Icon(menuiconname, size=16),
                _class="bx--overflow-menu__trigger"
                + (
                    " bx--tooltip__trigger bx--tooltip--a11y bx--tooltip--right bx--tooltip--align-start"
                    if menuname is not None
                    else ""
                ),
                aria_haspopup="true",
                aria_expanded="false",
                aria_controls=menuid,
                type="button",
                id=triggerid,
            ),
            hg.DIV(
                hg.UL(
                    hg.Iterator(
                        links,
                        "link",
                        hg.LI(
                            hg.F(asoverflowbutton),
                            **item_attributes,
                        ),
                    ),
                    _class="bx--overflow-menu-options__content",
                ),
                _class="bx--overflow-menu-options"
                + (" bx--overflow-menu--flip" if flip else ""),
                tabindex="-1",
                role="menu",
                aria_labelledby=triggerid,
                data_floating_menu_direction=direction,
                id=menuid,
            ),
            **attributes,
        )
        if menuname is not None:
            self[0].insert(0, hg.SPAN(menuname, _class="bx--assistive-text"))



# Generated by Django 4.1.2 on 2022-11-07 07:56

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="CustomForm",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=255, verbose_name="Title")),
                (
                    "pk_fields",
                    models.CharField(
                        blank=True,
                        help_text="If the form should be used to update items,\nthis fields specifies the fields that are used to\nfilter for the instance to update",
                        max_length=1024,
                        verbose_name="PK fields",
                    ),
                ),
                (
                    "model",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="contenttypes.contenttype",
                        verbose_name="Model",
                    ),
                ),
            ],
            options={
                "verbose_name": "Custom form",
                "verbose_name_plural": "Custom forms",
            },
        ),
        migrations.CreateModel(
            name="CustomFormField",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "fieldname",
                    models.CharField(
                        help_text="See help for list of possible fiel name",
                        max_length=1024,
                        verbose_name="Field name",
                    ),
                ),
                (
                    "label",
                    models.CharField(
                        blank=True,
                        help_text="Only use to override default label",
                        max_length=1024,
                        verbose_name="Label",
                    ),
                ),
                (
                    "help_text",
                    models.TextField(
                        blank=True,
                        help_text="Only use to override default help text",
                        verbose_name="Help text",
                    ),
                ),
                (
                    "customform",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="customformfields",
                        to="customforms.customform",
                        verbose_name="Custom form",
                    ),
                ),
            ],
            options={
                "verbose_name": "Custom form field",
                "verbose_name_plural": "Custom form fields",
            },
        ),
    ]



# Generated by Django 4.0.5 on 2022-09-13 09:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('triggers', '0004_datachangetrigger_field'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='datachangetrigger',
            options={'verbose_name': 'Change trigger', 'verbose_name_plural': 'Change triggers'},
        ),
        migrations.AlterModelOptions(
            name='datefieldtrigger',
            options={'verbose_name': 'Date trigger', 'verbose_name_plural': 'Date triggers'},
        ),
        migrations.AlterModelOptions(
            name='sendemail',
            options={'verbose_name': 'Send email action', 'verbose_name_plural': 'Send email actions'},
        ),
        migrations.AlterField(
            model_name='datachangetrigger',
            name='field',
            field=models.CharField(blank=True, help_text='Only trigger when a certain field has changed. Use comma to add multiple fields.', max_length=255, verbose_name='Field'),
        ),
    ]



# Generated by Django 4.0.5 on 2022-09-28 06:33

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('triggers', '0006_action_model'),
    ]

    operations = [
        migrations.AlterField(
            model_name='datachangetrigger',
            name='action',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='triggers.action'),
        ),
        migrations.AlterField(
            model_name='datefieldtrigger',
            name='action',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='triggers.action'),
        ),
        migrations.AlterField(
            model_name='datefieldtrigger',
            name='offset_amount',
            field=models.IntegerField(default=0, help_text='Can be negative (before) or positive (after)', verbose_name='Offset amount'),
        ),
        migrations.AlterField(
            model_name='datefieldtrigger',
            name='offset_type',
            field=models.CharField(choices=[('minutes', 'Minutes'), ('hours', 'Hours'), ('days', 'Days'), ('weeks', 'Weeks'), ('months', 'Months'), ('years', 'Years')], default='days', max_length=255, verbose_name='Offset type'),
        ),
    ]



# Generated by Django 4.0.5 on 2022-09-28 05:10

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('triggers', '0005_alter_datachangetrigger_options_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='action',
            name='model',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='contenttypes.contenttype', verbose_name='Model'),
        ),
    ]



import io

import htmlgenerator as hg
from django.apps import apps
from django.core.management import call_command
from django.forms import Form, formset_factory
from django.utils.translation import gettext_lazy as _

from basxbread import layout, utils
from basxbread.layout.components import forms

from .parser import FieldForm, ModelForm, field2formdata, model2formdata

# concept:
# get models and fields from django config? ==> create form
# let user edit form
# detect changes in form
# in form_success:
#     backup model files and database
#     use redbaron to write changes to model file
#     generate migration files
#     run migration

EDITABLE_APP = "customdata"


@utils.aslayout
def test(request):
    data = None
    if request.method == "POST":
        data = request.POST
    modelforms = {}
    for model in apps.get_app_config(EDITABLE_APP).get_models():
        prefix = model._meta.label_lower.replace(".", "-")
        modelforms[model] = [
            ModelForm(data=data, initial=model2formdata(model), prefix=prefix),
            formset_factory(FieldForm, extra=0)(
                data=data,
                initial=[field2formdata(f) for f in model._meta.get_fields()],
                prefix=prefix,
            ),
        ]
    if request.method == "POST":
        made_changes = False
        for model, (modelform, fieldforms) in modelforms.items():
            if modelform.is_valid() and modelform.changed_data:
                modelform.apply_changes(model)
                made_changes = True
            if fieldforms.is_valid():
                for fieldform in fieldforms:
                    if fieldform.changed_data:
                        fieldform.apply_changes(model)
                        made_changes = True
        if made_changes:
            message_out = io.StringIO()
            error_out = io.StringIO()
            # TODO: create migration files manually
            call_command(
                "makemigrations",
                EDITABLE_APP,
                interactive=False,
                stdout=message_out,
                stderr=error_out,
            )
            call_command(
                "migrate",
                EDITABLE_APP,
                interactive=False,
                stdout=message_out,
                stderr=error_out,
            )

    content = hg.DIV(
        *[
            layout.grid.Row(layout.grid.Col(forms.FormField(f)))
            if f == "type"
            else forms.FormField(f)
            for f in FieldForm().fields
        ],
        style=hg.format(
            "display: {}",
            hg.If(
                hg.F(
                    lambda c: c.get(
                        f"{layout.forms.DEFAULT_FORMSET_CONTEXTNAME}_index", -1
                    )
                    == 0
                ),
                "block",
                "none",
            ),
        ),
        id=hg.format(
            "field-{}",
            hg.C(layout.forms.DEFAULT_FORMSET_CONTEXTNAME).prefix,
        ),
        _class="field-formset",
    )

    return hg.DIV(
        hg.SCRIPT(
            hg.mark_safe(
                """
function add_newfield_tab(prefix){
    console.log($("#fieldformset-" + prefix + "-container div:nth-child(1)"));
    let container = $("#fieldformset-" + prefix + "-container div:nth-child(1)");
    let newbutton = $("#fieldformset-" + prefix + "-container div:nth-child(1) button").cloneNode(true);
    let addbutton = container.lastChild;
    let index = parseInt($("#id_" + prefix + "-TOTAL_FORMS").value) - 1
    newbutton.setAttribute("onclick", newbutton.getAttribute("onclick").replace(prefix + "-0", prefix + "-" + index));
    newbutton.innerText = "<New field>";
    container.appendChild(newbutton);
    container.appendChild(addbutton);
}
"""
            )
        ),
        hg.H1("ModelEdit test page"),
        hg.DIV(
            layout.forms.Form(
                Form(),
                hg.DIV(
                    layout.tabs.Tabs(
                        *[
                            layout.tabs.Tab(
                                label=model._meta.verbose_name,
                                content=hg.DIV(
                                    forms.Form(
                                        modelform,
                                        *[
                                            forms.FormField(f)
                                            for f in ModelForm().fields
                                        ],
                                        standalone=False,
                                    ),
                                    hg.H4(_("Fields")),
                                    layout.forms.Formset(
                                        fieldsformset, content=content
                                    ).management_form,
                                    hg.DIV(
                                        layout.tile.Tile(
                                            hg.Iterator(
                                                fieldsformset,
                                                "form",
                                                layout.button.Button(
                                                    hg.C("form").initial.get(
                                                        "name", _("<New field>")
                                                    ),
                                                    buttontype="ghost",
                                                    onclick=hg.format(
                                                        "$$('.field-formset')._.style({{display: 'none'}}); $('#field-{}')._.style({{display: 'block'}})",
                                                        hg.C("form").prefix,
                                                    ),
                                                ),
                                            ),
                                            layout.forms.Formset(
                                                fieldsformset, hg.BaseElement()
                                            ).add_button(
                                                f"#fieldformset-{modelform.prefix}-container div:nth-child(2)",
                                                label=_("Add new field"),
                                                onclick=hg.format(
                                                    "add_newfield_tab('{}')",
                                                    modelform.prefix,
                                                ),
                                                notext=False,
                                            ),
                                            style="display: flex; flex-direction: column",
                                        ),
                                        layout.tile.Tile(
                                            layout.forms.Formset(
                                                fieldsformset, content=content
                                            ),
                                            style="display: flex; flex-direction: column",
                                        ),
                                        id=f"fieldformset-{modelform.prefix}-container",
                                        style="display: grid; grid-template-columns: 20% 80%; column-gap: 5px",
                                    ),
                                ),
                            )
                            for model, (modelform, fieldsformset) in modelforms.items()
                        ]
                    )
                ),
                layout.forms.helpers.Submit(),
            )
        ),
    )



import datetime

import htmlgenerator as hg
from django.conf import settings
from django.db import models
from django.db.models import ManyToOneRel
from django.db.models.fields.related_descriptors import ReverseManyToOneDescriptor
from django.template.defaultfilters import linebreaksbr
from django.utils.formats import localize as djangolocalize
from django.utils.text import slugify as djangoslufigy
from django.utils.timezone import localtime as djangolocaltime

from ..formatters import as_download
from ..utils import resolve_modellookup

DEVMODE_KEY = "DEVMODE"


class HasBasxBreadCookieValue(hg.Lazy):
    def __init__(self, cookiename, value, default=None):
        self.cookiename = cookiename
        self.value = value
        self.default = default

    def resolve(self, context):
        if f"basxbread-{self.cookiename}" in context["request"].session.get(
            "basxbread-cookies", {}
        ):
            return (
                context["request"].session["basxbread-cookies"][
                    f"basxbread-{self.cookiename}"
                ]
                == self.value
            )
        return self.default == self.value


class DevModeOnly(hg.BaseElement):
    def render(self, context, stringify=True, fragment=None):
        if context["request"].session.get(DEVMODE_KEY, False):
            return super().render(context, stringify=stringify, fragment=fragment)
        return ""


def aslink_attributes(href):
    """
    Shortcut to generate HTMLElement attributes to make any element behave like a link.
    This should normally be used like this: hg.DIV("hello", \\*\\*aslink_attributes('google.com'))
    """
    return {
        "onclick": hg.BaseElement("document.location = '", href, "'"),
        "onauxclick": hg.BaseElement("window.open('", href, "', '_blank')"),
        "style": "cursor: pointer",
    }


class ObjectFieldLabel(hg.ContextValue):
    def __init__(self, fieldname, object_contextname="object", title=True):
        """
        :param fieldname: Name of the model field whose value will be rendered
        :param object_contextname: Name of the context object which provides the field value
                                   or a Lazy which produces the object itself
        """
        super().__init__(object_contextname)
        self.fieldname = fieldname
        self.title = title
        self.object = object_contextname

    def resolve(self, context):
        object = self.object
        if isinstance(self.object, str):
            object = resolve_modellookup(context, self.object)[0]
        object = hg.resolve_lazy(object, context)
        label = resolve_modellookup(object._meta.model, self.fieldname)[-1]
        label = getattr(
            object._meta.model.objects.get_queryset().query.annotations.get(
                self.fieldname, None
            ),
            "output_field",
            label,
        )  # test for annotated ("dynamic") fields

        if isinstance(label, ReverseManyToOneDescriptor):
            label = label.rel.field.remote_field

        if hasattr(label, "verbose_name"):
            return label.verbose_name
        if isinstance(label, property):
            return label.fget.__name__.replace("_", " ")
        if callable(label):
            return label.__name__.replace("_", " ")
        if isinstance(label, ManyToOneRel):
            return (
                label.related_model._meta.verbose_name_plural
            )  # this is "more correct", but not sure if it always works..
            # return label.name.replace("_", " ").capitalize()
        return label.title() if self.title and isinstance(label, str) else label


class ObjectFieldValue(hg.Lazy):
    def __init__(self, fieldname, object_contextname="object", formatter=None):
        """
        :param fieldname: Name of the model field whose value will be rendered
        :param object_contextname: Name of the context object which provides the field value
                                   or a Lazy which produces the object itself
        :param formatter: function which takes the field value as a single
                          argument and returns a formatted version
        """
        self.object = object_contextname
        self.fieldname = fieldname
        self.formatter = formatter

    def resolve(self, context):
        object = self.object
        if isinstance(self.object, str):
            object = resolve_modellookup(context, self.object)[0]
        object = hg.resolve_lazy(object, context)

        parts = self.fieldname.split(".")
        # test if the value has a matching get_FIELDNAME_display function
        try:
            value = hg.resolve_lookup(
                object, f"{'.'.join(parts[:-1])}.get_{parts[-1]}_display".lstrip(".")
            )
        except Exception:
            value = None
        if value is None:
            try:
                value = hg.resolve_lookup(object, self.fieldname)
            except AttributeError:
                # e.g. for non-existing OneToOneField related value
                pass

        if isinstance(value, models.fields.files.FieldFile) and not self.formatter:
            return as_download(value)

        if isinstance(value, datetime.datetime):
            value = djangolocaltime(value)

        if self.formatter:
            value = self.formatter(value)

        if isinstance(value, (type(None), bool)):
            value = {
                True: settings.HTML_TRUE,
                False: settings.HTML_FALSE,
                None: settings.HTML_NONE,
            }[value]

        value = djangolocalize(value, use_l10n=settings.USE_L10N)
        if isinstance(value, models.Manager):
            value = ", ".join([str(x) for x in value.all()])
        if isinstance(value, str):
            value = linebreaksbr(value)
        return value


localize = hg.lazify(djangolocalize)
localtime = hg.lazify(djangolocaltime)
slugify = hg.lazify(djangoslufigy)



import htmlgenerator as hg
from django import forms
from django.contrib.auth.forms import PasswordResetForm
from django.contrib.auth.models import Group, User
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from basxbread import layout, utils

from . import AddView, BrowseView, EditView, ReadView


class UserBrowseView(BrowseView):
    model = User
    columns = [
        "is_active",
        "username",
        "first_name",
        "last_name",
        "email",
        "is_superuser",
        "last_login",
    ]
    rowactions = (
        BrowseView.editlink(),
        utils.Link(
            href=utils.LazyHref("password_reset", query={"email": hg.C("row").email}),
            label=hg.If(
                hg.C("row").email, _("Send password reset"), _("User has no email")
            ),
            iconname="mail--all",
            is_submit=True,
            attributes={"disabled": hg.If(hg.C("row").email, None, True)},
            formfields={"email": hg.C("row").email},
            confirm_text=hg.format(
                _("Are you sure you want to send a password reset email to {}?"),
                hg.C("row").email,
            ),
        ),
    )


class UserEditView(EditView):
    model = User
    fields = [
        "is_active",
        "username",
        "first_name",
        "last_name",
        "email",
        "is_superuser",
        "groups",
        "user_permissions",
    ]

    layout = hg.DIV(
        hg.DIV(
            layout.forms.FormField("is_active"),
            layout.forms.FormField("username"),
            layout.forms.FormField("first_name"),
            layout.forms.FormField("last_name"),
            layout.forms.FormField("email"),
            layout.forms.FormField("is_superuser"),
            style="width: 100%",
        ),
        hg.DIV(
            layout.forms.FormField("groups"),
            layout.forms.FormField("user_permissions"),
            style="width: 100%",
        ),
        style="display: flex",
    )


class UserAddView(AddView):
    model = User
    fields = [
        "is_active",
        "username",
        "first_name",
        "last_name",
        "email",
        "is_superuser",
        "groups",
        "user_permissions",
    ]

    layout = hg.BaseElement(
        hg.DIV(
            hg.DIV(
                layout.forms.FormField("is_active"),
                layout.forms.FormField("username"),
                layout.forms.FormField("first_name"),
                layout.forms.FormField("last_name"),
                layout.forms.FormField("email"),
                layout.forms.FormField("is_superuser"),
                style="width: 100%",
            ),
            hg.DIV(
                layout.forms.FormField("groups"),
                layout.forms.FormField("user_permissions"),
                style="width: 100%",
            ),
            style="display: flex",
        ),
        hg.HR(),
        layout.forms.FormField("send_password_reset"),
    )

    def get_form_class(self):
        request = self.request

        class CustomForm(super().get_form_class()):
            send_password_reset = forms.BooleanField(
                required=False,
                label=_("Send password reset to user"),
                help_text=_(
                    "When checking this, a link will be sent to the user "
                    "which allows him to set his password"
                ),
            )

            def clean(self):
                clean = super().clean()
                if clean["send_password_reset"] and not clean["email"]:
                    raise ValidationError(
                        _(
                            "Please enter the user's email address in order "
                            "to send a password reset"
                        )
                    )
                return clean

            def save(self):
                ret = super().save()
                if self.cleaned_data["send_password_reset"]:
                    f = PasswordResetForm({"email": self.cleaned_data["email"]})
                    if f.is_valid():
                        f.save(request=request, use_https=request.is_secure())
                return ret

        return CustomForm


class UserReadView(ReadView):
    model = User
    fields = [
        "is_active",
        "username",
        "first_name",
        "last_name",
        "email",
        "is_superuser",
        "groups",
        "user_permissions",
    ]


class GroupBrowseView(BrowseView):
    model = Group


class GroupEditView(EditView):
    model = Group



# Generated by Django 4.0.5 on 2022-09-27 08:36

from django.db import migrations, models

import basxbread.contrib.publicurls.models


class Migration(migrations.Migration):

    dependencies = [
        ('publicurls', '0003_alter_publicurl_url'),
    ]

    operations = [
        migrations.AddField(
            model_name='publicurl',
            name='create_new_entry_from_response',
            field=models.CharField(blank=True, help_text="If 'Has Form' is set and the response is a redirect, the redirect URL is checked against this field (as regex). If it matches, a new public URL entry will automatically be created which allows the user to access the newly created and redirecte-to resource.", max_length=2048, validators=[basxbread.contrib.publicurls.models.validate_regex], verbose_name='Create new entry from response'),
        ),
    ]



from django.utils import formats
from django.utils.formats import localize
from django.utils.timezone import now
from jinja2.sandbox import SandboxedEnvironment


def jinja_env():
    environment = SandboxedEnvironment()
    environment.filters["map"] = lambda value, map: map.get(value, value)
    environment.filters["localize"] = localize
    environment.filters["date"] = formats.date_format
    environment.filters["time"] = formats.time_format
    environment.globals["now"] = now
    return environment


def jinja_render(template, **kwargs):
    return jinja_env().from_string(template).render(**kwargs)



import os
import signal

from django.http import HttpResponseRedirect

from .export import *  # noqa
from .jinja2 import *  # noqa
from .links import *  # noqa
from .model_helpers import *  # noqa
from .urls import *  # noqa


def get_all_subclasses(cls):
    return set(cls.__subclasses__()).union(
        [s for c in cls.__subclasses__() for s in get_all_subclasses(c)]
    )


# helper class to restart server after response has been sent
class HttpResponseRestartUWSGIServer(HttpResponseRedirect):
    def close(self):
        super().close()
        restart_uwsgi_appserver()


def restart_uwsgi_appserver():
    os.kill(os.getpid(), signal.SIGHUP)



import typing

import htmlgenerator as hg
from django.urls import path
from django.utils.translation import gettext_lazy as _

from basxbread import layout
from basxbread.utils import quickregister, reverse_model
from basxbread.views import AddView, EditView

from .models import DataChangeTrigger, DateFieldTrigger, SendEmail


def help_button(context):
    column_helper = layout.modal.Modal(
        _("Field explorer"),
        layout.fieldexplorer.field_help(context["object"].model.model_class()),
        size="lg",
    )
    return hg.BaseElement(
        layout.button.Button(
            _("Help"), buttontype="ghost", **column_helper.openerattributes
        ),
        column_helper,
    )


def field_help_button(context):
    column_helper = layout.modal.Modal(
        _("Available fields"),
        layout.fieldexplorer.field_help(context["object"].model.model_class(), 0),
        size="sm",
    )
    return hg.BaseElement(
        layout.button.Button(
            _("Show available fields"),
            buttontype="ghost",
            **column_helper.openerattributes
        ),
        column_helper,
    )


urlpatterns: typing.List[path] = []
quickregister(
    urlpatterns,
    DataChangeTrigger,
    editview=EditView._with(
        fields=[
            hg.H4(
                layout.ObjectFieldLabel("model"), ": ", layout.ObjectFieldValue("model")
            ),
            "description",
            "action",
            "type",
            "filter",
            hg.F(field_help_button),
            "field",
            "enable",
        ],
    ),
    addview=AddView._with(
        fields=["description", "model"],
        get_success_url=lambda s: reverse_model(
            s.model, "edit", kwargs={"pk": s.object.pk}
        ),
    ),
)
quickregister(
    urlpatterns,
    DateFieldTrigger,
    editview=EditView._with(
        fields=[
            hg.H4(
                layout.ObjectFieldLabel("model"), ": ", layout.ObjectFieldValue("model")
            ),
            "description",
            "action",
            "offset_type",
            "offset_amount",
            hg.F(help_button),
            "field",
            "filter",
            "enable",
        ]
    ),
    addview=AddView._with(
        fields=["description", "model"],
        get_success_url=lambda s: reverse_model(
            s.model, "edit", kwargs={"pk": s.object.pk}
        ),
    ),
)


quickregister(
    urlpatterns,
    SendEmail,
    editview=EditView._with(
        fields=[
            hg.H4(
                layout.ObjectFieldLabel("model"), ": ", layout.ObjectFieldValue("model")
            ),
            hg.F(help_button),
            "description",
            "email",
            "subject",
            "message",
        ]
    ),
    addview=AddView._with(
        fields=["description", "model"],
        get_success_url=lambda s: reverse_model(
            s.model, "edit", kwargs={"pk": s.object.pk}
        ),
    ),
)



import htmlgenerator as hg
from django import forms
from django.shortcuts import get_object_or_404
from django.utils.translation import gettext_lazy as _
from guardian.shortcuts import get_objects_for_user

from basxbread import formatters, layout, utils, views

from . import models


def formview_processing(request, form, initial=None):
    model = form.model.model_class()
    pk_fields = [f.strip() for f in form.pk_fields.split(",")]
    GET = request.GET.copy()
    filter_kwargs = {field: GET.pop(field)[0] for field in pk_fields if field in GET}
    instance = (
        get_objects_for_user(
            request.user,
            f"view_{model.__name__.lower()}",
            model.objects.filter(**filter_kwargs),
            with_superuser=True,
        )
        if form.pk_fields and filter_kwargs
        else model.objects.none()
    )
    view_class = views.AddView
    view_kwargs = {}
    if pk_fields:
        if instance.count() > 1:  # show an error
            return hg.BaseElement(
                layout.notification.InlineNotification(
                    _("Error"),
                    _("More than one instance was selected by the query elements"),
                    kind="error",
                ),
                _("The following arguments were passed to the query:"),
                layout.datatable.DataTable(
                    [
                        layout.datatable.DataTableColumn(
                            _("Query field name"), hg.C("row.0")
                        ),
                        layout.datatable.DataTableColumn(
                            _("Query field value"), hg.C("row.1")
                        ),
                    ],
                    filter_kwargs.items(),
                ).with_toolbar(title=_("Model: ") + model._meta.verbose_name),
                layout.datatable.DataTable.from_queryset(
                    instance, title=_("Produced results (should only produce 1)")
                ),
            )
        if instance.count() == 1:  # show an edit view
            view_class = views.EditView._with(success_url=request.get_full_path())
            view_kwargs["pk"] = instance.first().pk

    request.GET = GET
    initial = initial or {}
    return view_class._with(
        model=model,
        fields=[f.fieldname for f in form.customformfields.all()],
        initial=initial,
    ).as_view()(request, **view_kwargs)


@utils.aslayout
def formview(request, pk):
    form = get_object_or_404(models.CustomForm, pk=pk)
    return formview_processing(request, form=form)


class UploadForm(forms.Form):
    importfile = forms.FileField(required=False)


@utils.aslayout
def pdfimportview(request, pk):
    form = UploadForm()
    pdfimporter = get_object_or_404(models.PDFImport, pk=pk)
    if request.method == "POST":
        uploadform = UploadForm(request.POST, request.FILES)
        if uploadform.is_valid():
            if uploadform.cleaned_data.get("importfile"):
                initial = {}
                defined_formfields = {
                    f.pdf_field_name: f.customform_field.fieldname
                    for f in pdfimporter.fields.all()
                }
                for field, value in models.pdf_fields(
                    uploadform.cleaned_data["importfile"].read()
                ).items():
                    if field in defined_formfields:
                        if "." in defined_formfields[field]:
                            inlinefield, subfield = defined_formfields[field].split(
                                ".", 1
                            )
                            if inlinefield not in initial:
                                initial[inlinefield] = [{}]
                            if subfield in initial[inlinefield][-1]:
                                initial[inlinefield].append({})
                            initial[inlinefield][-1][subfield] = value
                        else:
                            initial[defined_formfields[field]] = value
                request.method = "GET"
                return formview_processing(
                    request, form=pdfimporter.customform, initial=initial
                )
            if "importfile" not in request.POST:
                return formview_processing(request, form=pdfimporter.customform)

    return hg.BaseElement(
        hg.H1(pdfimporter),
        hg.DIV(
            hg.format(
                _("Import filled out version of {}"),
                formatters.as_download(pdfimporter.pdf),
            ),
            style="margin-top: 2rem; margin-bottom: 2rem",
        ),
        layout.forms.Form(
            form,
            layout.forms.FormField("importfile"),
            layout.forms.helpers.Submit(label=_("Import")),
        ),
    )



# Generated by Django 4.1.2 on 2022-11-30 04:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("reports", "0013_fix_date_formatting"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="reportcolumn",
            name="sortingname",
        ),
    ]



# Generated by Django 4.1.5 on 2023-02-04 08:11

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("customforms", "0002_alter_customformfield_customform_pdfimport_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="pdfformfield",
            name="customform_field",
            field=models.ForeignKey(
                limit_choices_to={"customform": models.F("customform")},
                on_delete=django.db.models.deletion.CASCADE,
                to="customforms.customformfield",
            ),
        ),
    ]



import htmlgenerator as hg
from django.utils.translation import gettext_lazy as _

from basxbread import layout
from basxbread.utils import Link, ModelHref, link_with_urlparameters
from basxbread.views import BrowseView

from . import models


class TermsBrowseView(BrowseView):
    columns = [
        layout.datatable.DataTableColumn(
            models.Term._meta.get_field("term").verbose_name,
            hg.If(
                hg.C("row").disabled,
                hg.SPAN(hg.C("row").term, style="text-decoration: line-through;"),
                hg.C("row").term,
            ),
        ),
        "slug",
    ]

    def get_layout(self):
        ret = super().get_layout()
        has_disabled = "disabled" in self.request.GET
        url = link_with_urlparameters(
            self.request, **{"disabled": None if has_disabled else "true"}
        )
        ret[0].append(
            hg.DIV(
                hg.A(
                    hg.If(has_disabled, _("Hide disabled"), _("Show disabled")),
                    href=url,
                ),
                style="margin-top: 1rem",
            )
        )
        return ret

    def get(self, *args, **kwargs):
        vocabulary = models.Vocabulary.objects.filter(
            slug=self.request.GET.get("vocabulary_slug")
        ).first()
        if vocabulary:
            self.primary_button = layout.button.Button.from_link(
                Link(
                    href=ModelHref(
                        models.Term,
                        "add",
                        query={"vocabulary": vocabulary.id},
                        return_to_current=True,
                    ),
                    label=_("Add %s") % models.Term._meta.verbose_name,
                ),
                icon=layout.icon.Icon("add", size=20),
            )
        return super().get(*args, **kwargs)

    def get_queryset(self):
        qs = super().get_queryset()
        if "disabled" in self.request.GET:
            qs = self.model.objects.including_disabled()
        vocabulary_slug = self.request.GET.get("vocabulary_slug")
        if vocabulary_slug:
            qs = qs.filter(vocabulary__slug=vocabulary_slug)

        return qs


class VocabularyBrowseView(BrowseView):
    columns = ["name", "slug", "termcount"]
    rowactions = [
        BrowseView.editlink(),
        Link(
            href=ModelHref(
                models.Term, "browse", query={"vocabulary_slug": hg.C("row").slug}
            ),
            label=_("Terms of vocabulary"),
            iconname="tree-view--alt",
        ),
    ]



import datetime

from celery import shared_task
from django.apps import AppConfig
from django.conf import settings
from django.db import transaction
from django.utils import timezone
from kombu.utils.uuid import uuid

from .tasks import run_action

TRIGGER_PERIOD = getattr(settings, "TRIGGER_PERIOD", datetime.timedelta(hours=1))


class TriggersConfig(AppConfig):
    name = "basxbread.contrib.triggers"
    default_auto_field = "django.db.models.BigAutoField"

    def ready(self):
        from django.db.models.signals import post_save, pre_delete, pre_save

        from basxbread.utils.celery import RepeatedTask

        pre_save.connect(get_old_object)
        post_save.connect(save_handler)
        pre_delete.connect(delete_handler, dispatch_uid="trigger_delete")

        shared_task(
            base=RepeatedTask,
            run_every=TRIGGER_PERIOD,
            name="basxbread.contrib.triggers.apps.periodic_trigger",
        )(periodic_trigger)


# make sure we have access to the old value so we can change for field changes
def get_old_object(sender, instance, **kwargs):
    instance._old = None
    if instance.pk:
        try:
            instance._old = type(instance).objects.get(pk=instance.pk)
        except type(instance).DoesNotExist:
            pass


def save_handler(sender, instance, created, **kwargs):
    datachange_trigger(sender, instance, "added" if created else "changed")


def delete_handler(sender, instance, **kwargs):
    datachange_trigger(sender, instance, "deleted")


def datachange_trigger(model, instance, type):
    from django.contrib.contenttypes.models import ContentType

    from .models import DataChangeTrigger

    contenttypes = ContentType.objects.filter(
        app_label=model._meta.app_label, model=model._meta.model_name
    )
    for contenttype in contenttypes:
        for trigger in DataChangeTrigger.objects.filter(
            model=contenttype, type=type, enable=True
        ):
            if type == "changed" and trigger.field and instance._old is not None:
                fields = [i.strip() for i in trigger.field.split(",")]
                if all(  # if none of the fields has changed, skip this trigger
                    getattr(instance, field) == getattr(instance._old, field)
                    for field in fields
                ):
                    continue

            # the filter for new objects will be run in the on_commit code, as
            # we need the value to be inside the database
            if trigger.action and (
                trigger.filter.queryset.filter(pk=instance.pk).exists()
                or type == "added"
            ):
                name = f"Trigger '{trigger}': Action '{trigger.action}'"
                if type == "deleted":
                    # if we want to still have access to the database object
                    # while the action is performed, we need to execute the action
                    # immediately and cannot do it in the background with celery
                    run_action(trigger.action.pk, instance._meta.label, instance.pk)
                elif type == "changed":
                    transaction.on_commit(
                        lambda action_pk=trigger.action.pk: run_action.apply_async(
                            (action_pk, instance._meta.label, instance.pk),
                            shadow=name,
                            task_id=f"{name}-{uuid()}",
                        )
                    )
                elif type == "added":

                    def post_commit(thetrigger=trigger, theinstance=instance):
                        if thetrigger.filter.queryset.filter(
                            pk=theinstance.pk
                        ).exists():
                            run_action.apply_async(
                                (
                                    thetrigger.action.pk,
                                    theinstance._meta.label,
                                    theinstance.pk,
                                ),
                                shadow=name,
                                task_id=f"{name}-{uuid()}",
                            )

                    transaction.on_commit(post_commit)
    instance._old = None


def periodic_trigger():
    from .models import DateFieldTrigger

    print(
        f"Running trigger in range: {timezone.now()} - {timezone.now() + TRIGGER_PERIOD}"
    )
    for trigger in DateFieldTrigger.objects.filter(enable=True):
        for instance in trigger.filter.queryset.all():
            for td in trigger.triggerdates(instance):
                if (
                    td is not None
                    and timezone.now() <= td < timezone.now() + TRIGGER_PERIOD
                ) and trigger.action:
                    name = f"Trigger '{trigger}': Action '{trigger.action}'"
                    run_action.apply_async(
                        (trigger.action.pk, instance._meta.label, instance.pk),
                        shadow=name,
                        task_id=f"{name}-{uuid()}",
                    )



from celery import shared_task

from basxbread.utils import get_concrete_instance


@shared_task
def run_action(action_pk, modelname, instance_pk):
    from django.apps import apps

    from .models import Action

    get_concrete_instance(Action.objects.get(pk=action_pk)).run(
        apps.get_model(modelname).objects.get(pk=instance_pk)
    )



# Generated by Django 4.0.5 on 2022-10-01 08:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('triggers', '0007_alter_datachangetrigger_action_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='datefieldtrigger',
            name='offset_type',
            field=models.CharField(choices=[('seconds', 'Seconds'), ('minutes', 'Minutes'), ('hours', 'Hours'), ('days', 'Days'), ('weeks', 'Weeks'), ('months', 'Months'), ('years', 'Years')], default='days', max_length=255, verbose_name='Offset type'),
        ),
    ]



import ast
import inspect
import re

from django import forms
from django.core.exceptions import ValidationError
from django.db.models import Field, fields
from django.utils.translation import gettext_lazy as _

from basxbread.utils import get_all_subclasses


def validate_choices(value):
    val = ast.literal_eval(value)
    if not isinstance(val, (tuple, list)):
        raise ValidationError(
            _("%(value)s is not a list or tuple"), params={"value": value}
        )
    for i in val:
        if len(i) != 2:
            raise ValidationError(
                _("%(value)s does not only contain 2-tuples"), params={"value": value}
            )


def valid_ordering(value):
    val = ast.literal_eval(value)
    for i in val:
        if not isinstance(i, str):
            raise ValidationError(
                _("%(value)s does not only contain list of fields"),
                params={"value": value},
            )
        i = i[1:] if i.startswith("-") else i
        if not i.isidentifier():
            raise ValidationError(
                _("%(value)s does not only contain list of fields"),
                params={"value": value},
            )


def validate_identifier(value):
    if not value.isidentifier():
        raise ValidationError(
            _("%(value)s is not a valid Python identifier"), params={"value": value}
        )


def validate_literal(value):
    try:
        ast.literal_eval(value)
    except Exception:
        raise ValidationError(
            _("%(value)s is not a valid Python literal"), params={"value": value}
        )


class ModelForm(forms.Form):
    name = forms.CharField(
        label=_("Model name"),
        help_text=_(
            "Python identifier for the model (no spaces and special characters). Only used internally."
        ),
        required=True,
        max_length=255,
        strip=True,
        validators=[validate_identifier],
    )
    verbose_name = forms.CharField(
        label=_("Verbose name"),
        help_text=_("Name that will be used as label for the model."),
        required=True,
        max_length=255,
        strip=True,
    )
    verbose_name_plural = forms.CharField(
        label=_("Verbose name plural"),
        help_text=_("Name that will be used as label for the model in plural form."),
        required=True,
        max_length=255,
        strip=True,
    )
    ordering = forms.CharField(
        label=_("Ordering"),
        help_text=_(
            "List of space separated fields that determine the default ordering for the objects of this model type"
        ),
        max_length=1024,
        initial="[]",
        required=False,
        strip=True,
        validators=[validate_literal, valid_ordering],
    )
    order_with_respect_to = forms.CharField(
        label=_("Order with respect to"),
        help_text=_("Order with respect to foreign key field"),
        max_length=1024,
        required=False,
        strip=True,
        validators=[validate_literal],
    )

    def apply_changes(self, model):
        filename = inspect.getfile(model)
        with open(filename) as modelfile:
            source = modelfile.read()
            sourcelines = source.splitlines()
            a = ast.parse(source, filename=filename)
            for node in ast.walk(a):
                if isinstance(node, ast.ClassDef) and node.name == model.__name__:
                    classsource = sourcelines[node.lineno - 1][
                        node.col_offset : node.end_col_offset
                    ]
                    print(
                        re.sub(
                            rf"class {model.__name__}\((.*)\):",
                            rf"class {self.cleaned_data['name']}(\1)",
                            classsource,
                        )
                    )


def fieldtypename(fieldtype):
    return f"{fieldtype.__module__}.{fieldtype.__qualname__}"


def fieldtypes():
    return {
        fieldtypename(fieldtype): fieldtype for fieldtype in get_all_subclasses(Field)
    }


def typechoices():
    return tuple((value, type.__name__) for value, type in fieldtypes().items())


def typecoerce(value):
    if isinstance(value, str):
        return fieldtypes().get(value, None)
    return value


class DefaultField(forms.CharField):
    def _coerce(self, value):
        if isinstance(value, str):
            if value == "":
                return fields.NOT_PROVIDED
        return value

    def clean(self, value):
        value = super().clean(value)
        return self._coerce(value)


class FieldForm(forms.Form):
    name = forms.CharField(
        label=_("Field name"),
        help_text=_(
            "Python identifier for the field (no spaces and special characters). Only used internally."
        ),
        required=True,
        max_length=255,
        strip=True,
        validators=[validate_identifier],
    )
    type = forms.TypedChoiceField(
        choices=typechoices, coerce=typecoerce, empty_value=None
    )
    verbose_name = forms.CharField(
        label=_("Verbose name"),
        help_text=_("Name that will be used as label for the field."),
        required=False,
        max_length=255,
        strip=True,
    )
    null = forms.BooleanField(
        label=_("Null"),
        help_text=_("Allo this field to be null (not same as empty string)"),
        initial=False,
        required=False,
    )
    blank = forms.BooleanField(
        label=_("Blank"),
        help_text=_("Do not require this field to be filled out on forms"),
        initial=False,
        required=False,
    )
    editable = forms.BooleanField(
        label=_("Editable"),
        help_text=_("Allow this field to be edited via forms"),
        initial=True,
        required=False,
    )
    primary_key = forms.BooleanField(
        label=_("Primary key"),
        help_text=_("Make this field the primary key"),
        initial=False,
        required=False,
    )
    unique = forms.BooleanField(
        label=_("Unique"),
        help_text=_("Ensure this field is unique over all instances of the model"),
        initial=False,
        required=False,
    )
    help_text = forms.CharField(
        label=_("Help text"),
        help_text=_("Additional text to be displayed with the field"),
        max_length=1024,
        required=False,
    )
    default = DefaultField(
        label=_("Default"),
        help_text=_("Default value for this field, must be a Python literal"),
        required=False,
        max_length=1024,
        validators=[validate_literal],
    )
    choices = forms.CharField(
        label=_("Choices"),
        help_text=_(
            "Iterable of 2-tuples with (value, label) literals, generates a select input"
        ),
        required=False,
        max_length=1024,
        validators=[validate_literal, validate_choices],
    )

    # Special, depending on field:
    # max_length: int (BinaryField, CharField, EmailField, FileField)
    # auto_now: bool (DateField, DateTimeField)
    # auto_now_add: bool (DateField, DateTimeField)
    # max_digits: int (DecimalField)
    # decimal_places: int (DecimalField)
    # upload_to: path (FileField)

    def apply_changes(self, model):
        pass


def model2formdata(model):
    return {
        "name": model.__name__,
        "verbose_name": getattr(model._meta, "verbose_name"),
        "verbose_name_plural": getattr(model._meta, "verbose_name_plural"),
        "ordering": getattr(model._meta, "ordering") or "",
        "order_with_respect_to": getattr(model._meta, "order_with_respect_to"),
    }


def field2formdata(field):
    default = getattr(field, "default")
    return {
        "type": fieldtypename(type(field)),
        "name": getattr(field, "name"),
        "verbose_name": getattr(field, "verbose_name"),
        "null": getattr(field, "null"),
        "blank": getattr(field, "blank"),
        "editable": getattr(field, "editable"),
        "primary_key": getattr(field, "primary_key"),
        "unique": getattr(field, "unique"),
        "help_text": getattr(field, "help_text"),
        "default": "" if default == fields.NOT_PROVIDED else default,
        "choices": getattr(field, "choices"),
    }


def parse():
    return None



# Generated by Django 4.1.2 on 2022-11-09 07:02

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("customforms", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="customformfield",
            name="customform",
            field=models.ForeignKey(
                editable=False,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="customformfields",
                to="customforms.customform",
                verbose_name="Custom form",
            ),
        ),
        migrations.CreateModel(
            name="PDFImport",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "pdf",
                    models.FileField(upload_to="pdf_import", verbose_name="PDF form"),
                ),
                (
                    "customform",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="customforms.customform",
                    ),
                ),
            ],
            options={
                "verbose_name": "PDF import",
                "verbose_name_plural": "PDF imports",
            },
        ),
        migrations.CreateModel(
            name="PDFFormField",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "pdf_field_name",
                    models.CharField(max_length=256, verbose_name="PDF field name"),
                ),
                (
                    "customform_field",
                    models.ForeignKey(
                        limit_choices_to={
                            "customform": models.F("pdfimport__customform")
                        },
                        on_delete=django.db.models.deletion.CASCADE,
                        to="customforms.customformfield",
                    ),
                ),
                (
                    "pdfimport",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="fields",
                        to="customforms.pdfimport",
                        verbose_name="PDF form field",
                    ),
                ),
            ],
            options={
                "verbose_name": "PDF import field",
                "verbose_name_plural": "PDF import fields",
                "unique_together": {
                    ("pdfimport", "pdf_field_name"),
                    ("pdfimport", "customform_field"),
                },
            },
        ),
    ]



# Generated by Django 4.1.2 on 2022-11-25 10:49

from django.db import migrations


class Migration(migrations.Migration):
    def update_cell_templates(apps, _):
        for column in apps.get_model("reports.ReportColumn").objects.all():
            if column.cell_template and "value.format" in column.cell_template:
                column.cell_template = column.cell_template.replace(
                    "value.format", "value|date"
                )
                column.save()

    dependencies = [
        ("reports", "0012_reportcolumn_allow_html_and_more"),
    ]

    operations = [
        migrations.RunPython(update_cell_templates, migrations.RunPython.noop)
    ]



# Generated by Django 4.1.2 on 2023-01-13 04:03

from django.db import migrations, models

import basxbread.contrib.triggers.models


class Migration(migrations.Migration):

    dependencies = [
        ("triggers", "0009_alter_sendemail_email"),
    ]

    operations = [
        migrations.AlterField(
            model_name="datachangetrigger",
            name="field",
            field=models.CharField(
                blank=True,
                help_text="Only trigger when a certain field has changed. Use comma to add multiple fields.",
                max_length=255,
                validators=[
                    basxbread.contrib.triggers.models.validate_direct_attributes
                ],
                verbose_name="Field",
            ),
        ),
        migrations.AlterField(
            model_name="datefieldtrigger",
            name="field",
            field=models.CharField(
                help_text="The field of the selected model which should trigger an action",
                max_length=255,
                validators=[
                    basxbread.contrib.triggers.models.validate_direct_attributes
                ],
                verbose_name="Field",
            ),
        ),
    ]



import htmlgenerator as hg
from django.utils.translation import gettext_lazy as _

from basxbread import layout, menu, utils
from basxbread import views as breadviews

from . import models, views

urlpatterns = [
    *utils.default_model_paths(
        models.CustomForm,
        browseview=breadviews.BrowseView._with(
            columns=["__all__", "customformfields"],
            rowactions=(
                breadviews.BrowseView.editlink(),
                breadviews.BrowseView.deletelink(),
                utils.Link(
                    href=utils.ModelHref.from_object(
                        hg.C("row"), "use", return_to_current=False
                    ),
                    label=_("Add form entry"),
                    iconname="new-tab",
                ),
            ),
        ),
        addview=breadviews.AddView._with(default_success_page="edit"),
        editview=breadviews.EditView._with(
            fields=[
                "title",
                "model",
                "pk_fields",
                layout.forms.Formset.as_inline_datatable(
                    "customformfields", ["fieldname", "label", "help_text"]
                ),
            ],
            default_success_page="edit",
        ),
        use=views.formview,
    ),
    *utils.default_model_paths(
        models.PDFImport,
        browseview=breadviews.BrowseView._with(
            rowactions=(
                breadviews.BrowseView.editlink(),
                breadviews.BrowseView.deletelink(),
                utils.Link(
                    href=utils.ModelHref.from_object(
                        hg.C("row"), "use", return_to_current=False
                    ),
                    label=_("Import new PDF"),
                    iconname="new-tab",
                ),
            ),
        ),
        addview=breadviews.AddView._with(
            fields=["pdf", "customform"], default_success_page="edit"
        ),
        editview=breadviews.EditView._with(
            fields=[
                "pdf",
                "customform",
                layout.forms.Formset.as_inline_datatable(
                    "fields", ["pdf_field_name", "customform_field"]
                ),
            ],
            default_success_page="edit",
        ),
        use=views.pdfimportview,
    ),
]

group = menu.Group(
    label=models.CustomForm._meta.verbose_name_plural, iconname="license--draft"
)
menu.registeritem(
    menu.Item(
        group=group,
        link=utils.Link(
            href=utils.ModelHref(models.CustomForm, "browse"),
            label=models.CustomForm._meta.verbose_name_plural,
            permissions=[
                f"{models.CustomForm._meta.app_label}.view_{models.CustomForm._meta.model_name}"
            ],
        ),
    )
)
menu.registeritem(
    menu.Item(
        group=group,
        link=utils.Link(
            href=utils.ModelHref(models.PDFImport, "browse"),
            label=models.PDFImport._meta.verbose_name_plural,
            iconname="document--import",
            permissions=[
                f"{models.PDFImport._meta.app_label}.view_{models.PDFImport._meta.model_name}"
            ],
        ),
    )
)



import fitz
from django import forms
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _


class CustomForm(models.Model):
    title = models.CharField(_("Title"), max_length=255)
    model = models.ForeignKey(
        ContentType, on_delete=models.PROTECT, verbose_name=_("Model")
    )
    pk_fields = models.CharField(
        _("PK fields"),
        max_length=1024,
        blank=True,
        help_text=_(
            """If the form should be used to update items,
this fields specifies the fields that are used to
filter for the instance to update"""
        ),
    )

    def __str__(self):
        return self.title

    class Meta:
        verbose_name = _("Custom form")
        verbose_name_plural = _("Custom forms")


class CustomFormField(models.Model):
    customform = models.ForeignKey(
        CustomForm,
        on_delete=models.CASCADE,
        verbose_name=_("Custom form"),
        related_name="customformfields",
        editable=False,
    )
    fieldname = models.CharField(
        _("Field name"),
        max_length=1024,
        help_text=_("See help for list of possible fiel name"),
    )
    label = models.CharField(
        _("Label"),
        max_length=1024,
        help_text=_("Only use to override default label"),
        blank=True,
    )
    help_text = models.TextField(
        _("Help text"),
        help_text=_("Only use to override default help text"),
        blank=True,
    )
    help_text.formfield_kwargs = {"widget": forms.Textarea(attrs={"rows": 1})}

    def __str__(self):
        return self.fieldname

    class Meta:
        verbose_name = _("Custom form field")
        verbose_name_plural = _("Custom form fields")


def pdf_fields(pdffile):
    fields = {}
    pdf = fitz.Document(stream=pdffile)
    for page in pdf:
        widget = page.first_widget
        while widget:
            fields[widget.field_name] = widget.field_value
            widget = widget.next
    pdf.close()
    return fields


class PDFImport(models.Model):
    pdf = models.FileField(_("PDF form"), upload_to="pdf_import")
    customform = models.ForeignKey(CustomForm, on_delete=models.PROTECT)

    @cached_property
    def pdf_fields(self):
        with self.pdf.open() as file:
            return pdf_fields(file.read())

    def __str__(self):
        return _("PDF import for %s") % self.customform

    class Meta:
        verbose_name = _("PDF import")
        verbose_name_plural = _("PDF imports")


class PDFFormField(models.Model):
    pdfimport = models.ForeignKey(
        PDFImport,
        on_delete=models.CASCADE,
        verbose_name=_("PDF form field"),
        related_name="fields",
        editable=False,
    )
    pdf_field_name = models.CharField(_("PDF field name"), max_length=256)
    pdf_field_name.lazy_choices = lambda field, request, instance: [
        (f, f) for f in instance.pdf_fields.keys()
    ]
    customform_field = models.ForeignKey(
        CustomFormField,
        on_delete=models.CASCADE,
        limit_choices_to={"customform": models.F("customform")},
    )

    def __str__(self):
        return self.pdf_field_name

    class Meta:
        verbose_name = _("PDF import field")
        verbose_name_plural = _("PDF import fields")
        unique_together = [
            ["pdfimport", "customform_field"],
            ["pdfimport", "pdf_field_name"],
        ]



# Generated by Django 4.1.2 on 2022-10-11 03:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("triggers", "0008_alter_datefieldtrigger_offset_type"),
    ]

    operations = [
        migrations.AlterField(
            model_name="sendemail",
            name="email",
            field=models.CharField(
                help_text="\nSyntax:\n- Email: example@example.com\n- User/Group: @username\n- From object: object.manager.email\n\nMultiple values van be separated by comma , e.g.\nboss@example.com, @adminuser, @reviewteam, primary_email_address.email\n",
                max_length=255,
                verbose_name="Email",
            ),
        ),
    ]



# Generated by Django 4.1.2 on 2022-11-25 10:57

from django.db import migrations


class Migration(migrations.Migration):
    def update_cell_templates(apps, _):
        for column in apps.get_model(
            "document_templates.DocumentTemplateVariable"
        ).objects.all():
            if column.template and "value.format" in column.template:
                column.template = column.template.replace("value.format", "value|date")
                column.save()

    dependencies = [
        ("document_templates", "0005_documenttemplate_filename_template"),
    ]

    operations = [
        migrations.RunPython(update_cell_templates, migrations.RunPython.noop)
    ]
